﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using LogicNodesSDK.Logic.Examples;
using NUnit.Framework;
using LogicModule.ObjectModel;
using LogicModule.Nodes;
using LogicModule.Nodes.TestHelper;


namespace ExampleNodesTest
{
  public class AggregationTestCaseData
  {
    public static readonly Func<Aggregation, double> max = p => p.OutputMax.Value;
    public static readonly Func<Aggregation, double> min = p => p.OutputMin.Value;
    public static readonly Func<Aggregation, double> avg = p => p.OutputAverage.Value;
    public static readonly Func<Aggregation, double> sum = p => p.OutputSum.Value;

    public static IEnumerable MaxTestCases
    {
      get
      {
        yield return new TestCaseData(new List<double>{0, 0, 0}, max, 0).SetName("Max: All values set to 0");
        yield return new TestCaseData(new List<double>{-1, -21, 0, 32, 18, -42, 0, 39}, max, 39).SetName("Max: Mixed values positive, negative and zero");
        yield return new TestCaseData(new List<double>{double.MaxValue, 5, 3}, max, double.MaxValue).SetName("Max: MaxValue");
        yield return new TestCaseData(new List<double>{double.MinValue, 4, 182}, max, 182).SetName("Max: MinValue");
        
      }
    }

    public static IEnumerable MinTestCases
    {
      get
      {
        yield return new TestCaseData(new List<double>{0, 0, 0}, min, 0).SetName("Min: All values set to 0");
        yield return new TestCaseData(new List<double>{49, -211, -132, 85}, min, -211).SetName("Min: Mixed values");
        yield return new TestCaseData(new List<double>{491, -2511, -1132, -985, double.MinValue}, min, double.MinValue).SetName("Min: MinValue");
        yield return new TestCaseData(new List<double>{491, -2511, -1132, -985, double.MaxValue}, min, -2511).SetName("Min: MaxValue");
      }
    }
    
    public static IEnumerable AvgTestCases
    {
      get
      {
        yield return new TestCaseData(new List<double>{0, 0, 0}, avg, 0).SetName("Avg: All values set to 0");
        yield return new TestCaseData(new List<double>{5, 10, 75, 0, -45, 20, 30, -25}, avg, 8.75).SetName("Avg: Mixed values");
        yield return new TestCaseData(new List<double>{10, -10, 0}, avg, 0).SetName("Avg: Simple test");
      }
    }
    
    public static IEnumerable SumTestCases
    {
      get
      {
        yield return new TestCaseData(new List<double>{0, 0, 0}, sum, 0).SetName("Sum: All values set to 0");
        yield return new TestCaseData(new List<double>{1, 21, 32, 15}, sum, 69).SetName("Sum: All values are positive");
        yield return new TestCaseData(new List<double>{1, -54, 21, -82, 0, 32, 15}, sum, -67).SetName("Sum: Mixed values");
      }
    }
  }

  [TestFixture]
  public class AggregationTest
  {
    
    public AggregationTest()
    {
      context = TestNodeContext.Create();
    }

    private readonly INodeContext context;
    private Aggregation node;

    [SetUp]
    public void InitilizeNode()
    {
      node = new Aggregation(context);
    }

    [TearDown]
    public void DeleteNode()
    {
      node = null;
    }

    [TestCaseSource(typeof(AggregationTestCaseData), "MinTestCases", Category = "MaxOutputTests")]
    [TestCaseSource(typeof(AggregationTestCaseData), "MaxTestCases", Category = "MinOutputTests")]
    [TestCaseSource(typeof(AggregationTestCaseData), "AvgTestCases", Category = "AvgOutputTests")]
    [TestCaseSource(typeof(AggregationTestCaseData), "SumTestCases", Category = "SumOutputTests")]
    public void AnyOutputTests(IList<Double> inputs, Func<Aggregation, double> func, double expected)
    {
      node.InputCount.Value = inputs.Count;

      foreach (var item in inputs.Select((value, index) => new {value, index}))
      {
        node.Inputs[item.index].Value = item.value;
      }
      
      node.Execute();
      Assert.AreEqual(expected, func(node));
    }
  }
}